@@ -48,7 +48,7 @@ glibpth="$*"
 # /usr/lib/libdb.so.1, but that is not really meant for public consumption.
 #  XXX Revisit after perl 5.10 -- should we apply this to older Solaris
 # versions too?  (A.D. 11/2007).
-case "`uname -r`" in
+case "`$run uname -r`" in
 5.[0-9]) ;;
 *) ignore_versioned_solibs=y ;;
 esac
@@ -129,7 +129,7 @@ esac
 
 # Check that /dev/fd is mounted.  If it is not mounted, let the
 # user know that suid scripts may not work.
-mount | grep '^/dev/fd ' 2>&1 > /dev/null
+$run mount | grep '^/dev/fd ' 2>&1 > /dev/null
 case $? in
 0) ;;
 *)
@@ -146,8 +146,17 @@ esac
 
 # See if libucb can be found in /usr/lib.  If it is, warn the user
 # that this may cause problems while building Perl extensions.
-/usr/bin/ls /usr/lib/libucb* >/dev/null 2>&1
-case $? in
+found_libucb=''
+case "$run" in
+'') /usr/bin/ls /usr/lib/libucb* >/dev/null 2>&1
+    found_libucb=$?
+    ;;
+*)  $run /usr/bin/ls '/usr/lib/libucb*' >/dev/null 2>&1
+    found_libucb=$?
+    ;;
+esac
+
+case $found_libucb in
 0)
 	cat <<END >&4
 
@@ -173,7 +182,7 @@ esac
 make -v > make.vers 2>&1
 if grep GNU make.vers > /dev/null 2>&1; then
     tmp=`type make | awk '{print $NF}' | sed 's/[()]//g'`
-    case "`/usr/bin/ls -lL $tmp`" in
+    case "`${ls:-'/usr/bin/ls'} -lL $tmp`" in
     ??????s*)
 	    cat <<END >&2
 
@@ -220,7 +229,7 @@ cat > UU/cc.cbu <<'EOCBU'
 # Get gcc to share its secrets.
 echo 'int main() { return 0; }' > try.c
 	# Indent to avoid propagation to config.sh
-	verbose=`${cc:-cc} -v -o try try.c 2>&1`
+	verbose=`${cc:-cc} $ccflags -v -o try try.c 2>&1`
 
 # XXX TODO:  'specs' output changed from 'Reading specs from' in gcc-[23] to 'Using
 # built-in specs' in gcc-4.  Perhaps we should just use the same gcc test as
@@ -243,12 +252,12 @@ in your ${cc:-cc} command.  (Note that the trailing "/" is required.)
 
 END
 	    # Apparently not needed, at least for as 2.7 and later.
-	    # cc="${cc:-cc} -B/usr/ccs/bin/"
+	    # cc="${cc:-cc} $ccflags -B/usr/ccs/bin/"
 	fi
 
 	# See if ld(1) is GNU ld(1).  GNU ld(1) might not work for this job.
 	# Recompute $verbose since we may have just changed $cc.
-	verbose=`${cc:-cc} -v -o try try.c 2>&1 | grep ld 2>&1`
+	verbose=`${cc:-cc} $ccflags -v -o try try.c 2>&1 | grep ld 2>&1`
 
 	if echo "$verbose" | grep ' /usr/ccs/bin/ld ' >/dev/null 2>&1; then
 	    # Ok, gcc directly calls the Solaris /usr/ccs/bin/ld.
@@ -271,7 +280,7 @@ END
 	    # (This may all depend on local configurations too.)
 
 	    # Recompute verbose with -Wl,-v to find GNU ld if present
-	    verbose=`${cc:-cc} -Wl,-v -o try try.c 2>&1 | grep /ld 2>&1`
+	    verbose=`${cc:-cc} $ccflags -Wl,-v -o try try.c 2>&1 | grep /ld 2>&1`
 
 	    myld=`echo $verbose | awk '/\/ld/ {print $1}'`
 	    # This assumes that gcc's output will not change, and that
@@ -323,9 +332,9 @@ int main() {
 return(0);
 }
 EOM
-	tryworkshopcc="${cc:-cc} try.c -o try"
+	tryworkshopcc="${cc:-cc} $ccflags try.c -o try"
 	if $tryworkshopcc >/dev/null 2>&1; then
-		cc_name=`./try`
+		cc_name=`$run ./try`
 		if test "$cc_name" = "workshop"; then
 			ccversion="`${cc:-cc} -V 2>&1|sed -n -e '1s/^[Cc][Cc]: //p'`"
 		fi
@@ -468,9 +477,9 @@ case "$uselargefiles" in
 ''|$define|true|[yY]*)
 
 # Keep these in the left margin.
-ccflags_uselargefiles="`getconf LFS_CFLAGS 2>/dev/null`"
-ldflags_uselargefiles="`getconf LFS_LDFLAGS 2>/dev/null`"
-libswanted_uselargefiles="`getconf LFS_LIBS 2>/dev/null|sed -e 's@^-l@@' -e 's@ -l@ @g'`"
+ccflags_uselargefiles="`$run getconf LFS_CFLAGS 2>/dev/null`"
+ldflags_uselargefiles="`$run getconf LFS_LDFLAGS 2>/dev/null`"
+libswanted_uselargefiles="`$run getconf LFS_LIBS 2>/dev/null|sed -e 's@^-l@@' -e 's@ -l@ @g'`"
 
     ccflags="$ccflags $ccflags_uselargefiles"
     ldflags="$ldflags $ldflags_uselargefiles"
@@ -487,7 +496,7 @@ case "$usemorebits" in
 	;;
 esac
 
-if test `uname -p` = i386; then
+if test `$run uname -p` = i386; then
     case "$use64bitint" in
     "$define"|true|[yY]*)
             ccflags="$ccflags -DPTR_IS_LONG"
@@ -495,13 +504,13 @@ if test `uname -p` = i386; then
     esac
 fi
 
-if test `uname -p` = sparc -o `uname -p` = i386; then
+if test `$run uname -p` = sparc -o `$run uname -p` = i386; then
     cat > UU/use64bitint.cbu <<'EOCBU'
 # This script UU/use64bitint.cbu will get 'called-back' by Configure
 # after it has prompted the user for whether to use 64 bit integers.
 case "$use64bitint" in
 "$define"|true|[yY]*)
-	    case "`uname -r`" in
+	    case "`$run uname -r`" in
 	    5.[0-4])
 		cat >&4 <<EOM
 Solaris `uname -r|sed -e 's/^5\./2./'` does not support 64-bit integers.
@@ -536,7 +545,7 @@ EOCBU
 # after it has prompted the user for whether to be maximally 64 bitty.
 case "$use64bitall-$use64bitall_done" in
 "$define-"|true-|[yY]*-)
-	    case "`uname -r`" in
+	    case "`$run uname -r`" in
 	    5.[0-6])
 		cat >&4 <<EOM
 Solaris `uname -r|sed -e 's/^5\./2./'` does not support 64-bit pointers.
@@ -545,7 +554,7 @@ EOM
 		exit 1
 		;;
 	    esac
-	    processor=`uname -p`;
+	    processor=`$run uname -p`;
 	    if test "$processor" = sparc; then
 		libc='/usr/lib/sparcv9/libc.so'
 		if test ! -f $libc; then
@@ -561,7 +570,7 @@ EOM
 	    case "${cc:-cc} -v 2>/dev/null" in
 	    *gcc*)
 		echo 'int main() { return 0; }' > try.c
-		case "`${cc:-cc} -mcpu=v9 -m64 -S try.c 2>&1 | grep 'm64 is not supported by this configuration'`" in
+		case "`${cc:-cc} $ccflags -mcpu=v9 -m64 -S try.c 2>&1 | grep 'm64 is not supported by this configuration'`" in
 		*"m64 is not supported"*)
 		    cat >&4 <<EOM
 
@@ -584,8 +593,8 @@ EOM
 		# This adds in -Wa,-xarch=v9.  I suspect that's superfluous,
 		# since the -m64 above should do that already.  Someone
 		# with gcc-3.x.x, please test with gcc -v.   A.D. 20-Nov-2003
-#		if test $processor = sparc -a X`getconf XBS5_LP64_OFF64_CFLAGS 2>/dev/null` != X; then
-#		    ccflags="$ccflags -Wa,`getconf XBS5_LP64_OFF64_CFLAGS 2>/dev/null`"
+#		if test $processor = sparc -a X`$run getconf XBS5_LP64_OFF64_CFLAGS 2>/dev/null` != X; then
+#		    ccflags="$ccflags -Wa,`$run getconf XBS5_LP64_OFF64_CFLAGS 2>/dev/null`"
 #		fi
 		ldflags="$ldflags -m64"
 
@@ -601,9 +610,9 @@ EOM
 		esac
 		;;
 	    *)
-		getconfccflags="`getconf XBS5_LP64_OFF64_CFLAGS 2>/dev/null`"
-		getconfldflags="`getconf XBS5_LP64_OFF64_LDFLAGS 2>/dev/null`"
-		getconflddlflags="`getconf XBS5_LP64_OFF64_LDFLAGS 2>/dev/null`"
+		getconfccflags="`$run getconf XBS5_LP64_OFF64_CFLAGS 2>/dev/null`"
+		getconfldflags="`$run getconf XBS5_LP64_OFF64_LDFLAGS 2>/dev/null`"
+		getconflddlflags="`$run getconf XBS5_LP64_OFF64_LDFLAGS 2>/dev/null`"
 		echo "int main() { return(0); } " > try.c
 		case "`${cc:-cc} $getconfccflags try.c 2>&1 | grep 'deprecated'`" in
 		*" -xarch=generic64 is deprecated, use -m64 "*)
